#!/usr/bin/perl

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.



#$Id: RegisterService.pl 86 2007-10-10 08:05:41Z sletort $

=head1 NAME

pmb_RegisterService.pl - Script to register a new BioMOBY Service

=head1 SYNOPSIS

pmb_RegisterService.pl --central_conf="list_of_central_config_files" --service_conf="list_of_service_config_files" --module=path/to/module.pm

=head1 DESCRIPTION

 This script permit to register a new BioMOBY service 
 using 2 types of configuration files:
 
 * central_conf:
    This file contains where to register your service,
    which mobycentral
    You can give a list of config files to register
    many centrals in one shot !
 
 * service_conf:
    This file contains information on the service,
    like, name, type, description, inputs, outputs,
    secondaries.
    You can give a list of config files: in that case,
    if a key is duplicated, the last value (in the 
    same order as your file list) overwrite the previous.
    
  * module
    The Perl module containing your Service Sub

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr

=head1 SUBROUTINES

=cut

my $O_PARAM = '';

BEGIN
{
    if (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }

    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
    if (-d "$dirprg/../lib/Appli")
    {
        unshift(@INC, "$dirprg/../lib/Appli");
    }
}

use strict;

#use lib '/bioinfo/www/bioinfo/services/biomoby/cgi-bin/Services/LIPM/lib/';
use MOBY::Client::Central;
use Getopt::Long;
use ParamParser;
use BiomobyUtils;
use Cwd;
use HTML::Entities;

MAIN:
{
use Data::Dumper;


    $O_PARAM = New ParamParser('GETOPTLONG', ("central_conf=s", "service_conf=s", "module=s", "special_conf=s"));

    &ControlUsage('central_conf');
    &ControlUsage('service_conf');
    &ControlUsage('module');
    $ENV{PATH_WS_BIOMOBY} = $O_PARAM->Get('path_ws_biomoby') if ($O_PARAM->IsDefined('path_ws_biomoby'));
    $ENV{URL_WS_BIOMOBY} = $O_PARAM->Get('url_ws_biomoby') if ($O_PARAM->IsDefined('url_ws_biomoby'));
    
    if ($O_PARAM->IsDefined('special_conf'))
    {
        &OverwriteServiceCfg();
        
        $O_PARAM->AssertFileExists('special_conf');
        $O_PARAM->Update($O_PARAM->Get('special_conf'), 'O');
        
        

        if ($O_PARAM->IsDefined('path_ws_biomoby'))
        {
            $ENV{PATH_WS_BIOMOBY} = $O_PARAM->Get('path_ws_biomoby');
        }
        if ($O_PARAM->IsDefined('url_ws_biomoby'))
        {
            $ENV{URL_WS_BIOMOBY} = $O_PARAM->Get('url_ws_biomoby');
        }
    }

    if ((!defined $ENV{PATH_WS_BIOMOBY}) || (!defined $ENV{URL_WS_BIOMOBY}))
    {
        &Exit(
            "ERROR: Environnement variable PATH_WS_BIOMOBY and URL_WS_BIOMOBY have to be set\nYou can use a special.cfg file to define thes values or your shell\n"
            );
    }
    else
    {

        print STDERR "PATH_WS_BIOMOBY: $ENV{PATH_WS_BIOMOBY}\n";
        print STDERR "URL_WS_BIOMOBY: $ENV{URL_WS_BIOMOBY}\n";

    }

    my $ra_input_articles  = &GetPrimaryArticles('iar');
    my $ra_output_articles = &GetPrimaryArticles('oar');

    my $rh_secondary = &GetSecondaryArticles();

    my $module_name_long = $O_PARAM->Get("module");
    (my $module_name) = ($module_name_long =~ /(\w+)\.pm/);

    #Sebastien 20/07/05
    #Je rajoute le repertoire parent du module dans le fichier du dispatcher pour
    #eviter d'avoir un module qui fait les eval et avoir tous les .pm
    #dans un repertoire lib

    my $module_directory = $module_name_long;
    $module_directory =~ s/$module_name\.pm$//;

    if ($module_directory eq '')
    {
        $module_directory = './';
    }
    my $module_abs_path = Cwd::abs_path($module_directory);

    my $services_dir = $ENV{PATH_WS_BIOMOBY} . '/services';
    $module_abs_path =~ s/^$services_dir//;
 
    $module_abs_path = '/' if ($module_abs_path eq '');
    
    my @a_path_to_module = split(/\//, $module_abs_path);
    
    push (@a_path_to_module, '') if ( $#a_path_to_module <0 );
    $module_directory = $a_path_to_module[$#a_path_to_module] . "/";
    
    $module_directory = '' if (! defined $module_directory);

    my $ra_registry_names = &executeRegistering($ra_input_articles, $ra_output_articles, $rh_secondary, $module_directory);

    my $service_name   = $O_PARAM->Get("service_name");
    my $dispatcher_url = $O_PARAM->Get("dispatcher_url");

    foreach my $registry_name (@{$ra_registry_names})
    {
    	&AddToDispatcher($dispatcher_url, $service_name, $module_name, $registry_name, $a_path_to_module[$#a_path_to_module]);
    }

}

=head2 function OverwriteServiceCfg

 Title        : OverwriteServiceCfg
 Usage        : &OverwriteServiceCfg()
 Prerequisite : none
 Function     : Overwrite parameters in service_conf file if special_conf is given
 Returns      : none
 Args         : none
 Globals      : $O_PARAM

=cut


sub OverwriteServiceCfg
{
    print STDERR "Overwrite " . $O_PARAM->Get('service_conf') . "\n";
    my $o_param = New ParamParser($O_PARAM->Get('service_conf'));
    $o_param->Update($O_PARAM->Get('special_conf'), 'O');
    $o_param->Dump($O_PARAM->Get('service_conf'));
}

=head2 function GetPrimaryArticles

 Title        : GetPrimaryArticles
 Usage        : &GetPrimaryArticles('IAR')
 Prerequisite : none
 Function     : Parse config file to get Prymary(ies) Article(s)
 Returns      : $ra_primary_articles INPUT parameter for  MOBY::Client::Central->registerService
 Args         : $what [IAR|OAR] defining Input ARticle | Output ARticle
 Globals      : $O_PARAM
 Warning      : For COLLECTION articles, the RDF document is not yet 
 		properly formatted. Moby-Dev Core group is working on this
		problem (see mailing betwenn M. Wilkinson & S. Carrere.
		Registering succeed but information are uncomplete.

=cut

sub GetPrimaryArticles
{
    my $what = shift();

    my @a_primary_articles_ns = $O_PARAM->GetKeys($what . '_?\d+_tp');

    my @a_articles = ();
    foreach my $primary_articles_ns (@a_primary_articles_ns)
    {
        my ($ns) = ($primary_articles_ns =~ /($what\_?\d+_)tp$/);
        $O_PARAM->SelectNameSpace($ns);

        #IAR1_TP=COLLECTION
        #IAR1_1_OCL=FASTA
        #IAR1_1_ONS=
        #IAR1_1_ONM=sequence_databank

        my $article_type = $O_PARAM->Get("tp");
        my $article_name = $O_PARAM->Get("nm");

        if ($article_type =~ /collection/i)
        {
            my @a_collection_objects_ns = $O_PARAM->GetKeys('\d+_otp');
            my @a_list_of_objects       = ();
            foreach my $collection_object_ns (@a_collection_objects_ns)
            {
                my ($object_ns) = ($collection_object_ns =~ /(\d+_)otp$/);
                $O_PARAM->SelectNameSpace('');
                $O_PARAM->SelectNameSpace("$ns$object_ns");
                my $object_type         = $O_PARAM->Get("otp");
                my @a_object_namespaces = split(',', $O_PARAM->Get("ons"));

                my %h_object_type_namespace = ();
                $h_object_type_namespace{$object_type} =
                  \@a_object_namespaces;    #creation of an hash : objType3 => \@namespaces

                my @a_object_def = (%h_object_type_namespace);

                push(@a_list_of_objects, \@a_object_def);

            }
            my @a_article_def = ($article_name, \@a_list_of_objects);
            push(@a_articles, \@a_article_def);

        }
        elsif ($article_type =~ /simple/i)
        {
            my $object_type = $O_PARAM->Get('otp');
if( $object_type eq '' )
{	$object_type = $O_PARAM->Get('1_otp');	}

            my @a_object_namespaces = split(',', $O_PARAM->Get('1_ons'));

            #[articleName1,[objType1 => \@namespaces]]

            my %h_object_type_namespace = ($object_type => \@a_object_namespaces);
            my @a_object_def            = (%h_object_type_namespace);
            my @a_article_def           = ($article_name, \@a_object_def);
            push(@a_articles, \@a_article_def);
        }

        else
        {
            &PrintUsage();
            &Exit("$article_type unknown for $ns" . "_tp parameter ... [collection|simple]\n");
        }

        $O_PARAM->SelectNameSpace('');
    }

    return (\@a_articles);
}

=head2 function GetSecondaryArticles

 Title        : GetSecondaryArticles
 Usage        : &GetSecondaryArticles()
 Prerequisite : none
 Function     : Parse config file to get Prymary(ies) Article(s)
 Returns      : $rh_secondary_articles INPUT optional parameter (secondary) for  MOBY::Client::Central->registerService
 Args         : none
 Globals      : $O_PARAM

=cut

sub GetSecondaryArticles
{
    my @a_secondary_articles_ns = $O_PARAM->GetKeys('sar\d+_name');

    my %h_secondary_articles = ();

    foreach my $secondary_article_ns (@a_secondary_articles_ns)
    {
        my ($ns) = ($secondary_article_ns =~ /(sar\d+_)name$/);
        $O_PARAM->SelectNameSpace($ns);

        my $article_name        = $O_PARAM->Get("name");
        my $article_description = $O_PARAM->Get("description");
        $article_description = HTML::Entities::encode_entities($article_description);
        my $article_type        = $O_PARAM->Get("type");
        my $article_default     = $O_PARAM->Get("default");
        my @a_article_enum      = split(',', $O_PARAM->Get("enum"));
        my $article_min         = $O_PARAM->Get("min");
        my $article_max         = $O_PARAM->Get("max");

        my %h_article_param = (
                               datatype    => $article_type,
                               description => $article_description,
                               default     => $article_default,
                               max         => $article_max,
                               min         => $article_min,
                               enum        => \@a_article_enum
                               );
        $h_secondary_articles{$article_name} = \%h_article_param;

        $O_PARAM->SelectNameSpace('');
    }

    return (\%h_secondary_articles);
}

=head2 procedure executeRegistering

 Title        : executeRegistering
 Usage        : &executeRegistering($ra_input_articles,$ra_output_articles,$rh_secondary)
 Prerequisite : none
 Function     : Execute service registering on each mobycentral given as parameters
 Returns      : \@a_registry_names: list of registries names where the service has been registered
 Args         : $ra_input_articles Array ref for Input articles definitions
 		$ra_output_articles Array ref for Output articles definitions
		$rh_secondary Hash ref for Secondary articles definitions
        $module_directory: for rdf file
 Globals      : $O_PARAM

=cut

sub executeRegistering
{
    my ($ra_input_articles, $ra_output_articles, $rh_secondary, $module_directory) = @_;

    my $service_name        = $O_PARAM->Get("service_name");
    my $service_type        = $O_PARAM->Get("service_type");
    my $service_category    = $O_PARAM->Get("service_category");
    my $auth_uri            = $O_PARAM->Get("auth_uri");
    my $service_contact     = $O_PARAM->Get("service_contact");
    my $dispatcher_url      = $O_PARAM->Get("dispatcher_url");
    my $service_description = $O_PARAM->Get("service_description");

    &LookForSpecialCharacters($O_PARAM, 'service_description');
    $service_description = HTML::Entities::encode($service_description);
    
    my @a_central_conf = split(/\s+/, $O_PARAM->Get("central_conf"));

    my @a_registry_names = ();

    foreach my $central_conf (@a_central_conf)
    {

        #$O_PARAM->SelectNameSpace($central_conf);
        $O_PARAM->Update($central_conf, 'A');
        my $mobycentral_name = $O_PARAM->Get("mobycentral_name");
        &SetMobyCentralENV($O_PARAM, $central_conf);

        # >>>>>>>  ICI POUR LE RDF <<<<<<<<

        my $signature_path = $O_PARAM->Get("signature_path") . '/';
        my $signature_url  = $O_PARAM->Get("signature_url");
        $signature_url =~ s/\/$//g;
        $signature_url .= '/';
        $module_directory = '' if ($module_directory eq '/');
        # >>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<

        my $moby_instance = MOBY::Client::Central->new();
        my ($register_call) =
          $moby_instance->registerService(
                                          serviceName  => $service_name,
                                          serviceType  => $service_type,
                                          authURI      => $auth_uri,
                                          contactEmail => $service_contact,
                                          description  => $service_description,
                                          category     => $service_category,
                                          URL          => $dispatcher_url,
                                          input        => $ra_input_articles,
                                          output       => $ra_output_articles,
                                          secondary    => $rh_secondary,
                                          signatureURL => $signature_url . $module_directory . $mobycentral_name . '_' . $service_name . ".rdf"
                                          );

        if ($register_call->success)
        {
            print STDOUT "Registering success for \"$mobycentral_name\" central\n";
            if (!-d $signature_path . $module_directory)
            {
                mkdir($signature_path . $module_directory);
            }

            my $rdf_content = $register_call->RDF;
            
            $rdf_content =~ s/^<!\[CDATA\[//;
            $rdf_content =~ s/\]\]>$//;
            $rdf_content =~ s/\n$//;

            open(RDF, ">" . $signature_path . $module_directory . $mobycentral_name . '_' .$service_name . ".rdf")
              || die("$!:" . $signature_path . $module_directory . $mobycentral_name . '_' .$service_name . ".rdf");
            print RDF $rdf_content;
            close RDF;
            print STDOUT "RDF file created " . $signature_path . $module_directory . $mobycentral_name . '_' .$service_name . ".rdf\n";

            push (@a_registry_names,$mobycentral_name);
        }

        else
        {
            print STDOUT "Registering failure for \"$mobycentral_name\" central: ", $register_call->message, "\n";
            exit;

            #foreach my $key (keys (%{$register_call}))
            #{
            #	if (defined $register_call->{$key})
            #	{
            #		print STDOUT "\t$key: $register_call->{$key}\n";
            #	}
            #}
        }

        $O_PARAM->SelectNameSpace('');
    }

    return \@a_registry_names;
}

=head2 procedure LookForSpecialCharacters

 Title        : LookForSpecialCharacters
 Usage        : &LookForSpecialCharacters($string_to_analyse)
 Prerequisite : 
 Function     : Look for special characters in a string (e.g: '[' is not allowed in
                service description)
 Returns      : 
 Args         : $param_to_analyse
 Globals      : $O_PARAM

=cut

sub LookForSpecialCharacters
{
    my ($O_PARAM, $param_to_analyze) = (@_);

    my @a_forbidden_characters = ('\[', '\]');

    foreach my $forbidden_character (@a_forbidden_characters)
    {
        if ($O_PARAM->Get($param_to_analyze) =~ /$forbidden_character/)
        {

            print STDOUT <<END;



Registering failure: configuration file is not valid.
$param_to_analyze contains forbidden character "$forbidden_character".

List of forbidden characters: @a_forbidden_characters

END

            exit;
        }
    }

    return;
}

=head2 procedure ControlUsage

 Title        : ControlUsage
 Usage        : &ControlUsage()
 Prerequisite : none
 Function     : Control usage of program
 Returns      : undef if OK 
 Args         : $param_name Param to control
 Globals      : $O_PARAM

=cut

sub ControlUsage
{
    my $param_name = shift();

    if (!(defined($O_PARAM->Get($param_name))) || ($O_PARAM->Get($param_name) eq ''))
    {
        &PrintUsage();
        &Exit("ERROR: $param_name is mandatory");
    }
    else
    {
        my @a_param_values = split(/\s+/, $O_PARAM->Get($param_name));
        foreach my $param_value (@a_param_values)
        {
            if (!(-e $param_value))
            {
                &PrintUsage();
                &Exit("ERROR: $param_value does not exist");
            }
            elsif (-z ($param_value))
            {
                &PrintUsage();
                &Exit("ERROR: $param_value is empty");
            }
            else
            {
                if ($param_name eq 'central_conf')
                {
                    return;
                }
                else
                {
                    $O_PARAM->Update($param_value, 'O');
                }

                if (($param_name eq 'module') && ($#a_param_values > 0))
                {
                    &PrintUsage();
                    &Exit("ERROR: $param_name parameter must contain only one value");
                }
            }
        }
    }
    return;
}

=head2 procedure PrintUsage

 Title        : PrintUsage
 Usage        : &PrintUsage()
 Prerequisite : none
 Function     : print usage of the program
 Returns      : none 
 Args         : none
 Globals      : none

=cut

sub PrintUsage
{
    print STDERR <<END;


Usage: 

$0 --central_conf="<CENTRAL_CONFIG_FILE#1> <CENTRAL_CONFIG_FILE#2>\" --service_conf=\"<SERVICE_CONFIG_FILE#1> <SERVICE_CONFIG_FILE#2>\" --module=path/to/module.pm [--special_conf=<specific_values>]>

	--central_conf: configuration file for mobycentral connection
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/central.conf
			If a list of file is given as input, duplicate keys are overwritten
			
	--service_conf: configuration file for service specifications
			* see example at: http://lipm-svn.toulouse.inra.fr/svn/biomoby/trunk/templates/service.conf
			If a list of file is given as input, duplicate keys are overwritten
	
  
	--module:  The Perl module containing your Service Sub

Optionnal:    

    --special_conf: configuration file for current values (RDF path file, service_contact, dispatcher url

	
END
}
