# 
# 	sebastien.letort@toulouse.inra.fr
#	Created: May 09, 2007
#	Last Updated: september 20, 2007
#

package Error::ObjectException;

=pod

=head1 NAME

	Error::ObjectException - a class to define specific exception of Objects

=head1 SYNOPSIS

	throw Error::ObjectException( $class_name, KEYWORD, -text => $msg );
	throw Error::ObjectException( $obj, KEYWORD, -text => $msg );
	catch Error::ObjectException with {...}

=head1 DESCRIPTION

	With this class you can describe all type of exception encountered by object.

	- try to access to an attribute that doesn't exist
	- an attribute doesn't have a correct value

=head1 AUTHOR

Sebastien Letort : sebastien.letort@toulouse.inra.fr

=cut

use warnings;
use strict;

## this class inherit from LipmError.pm
use base qw(Error::LipmError);

BEGIN
{
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

use constant CAN_T_BE_INSTANCIED	=>	1;
use constant UNKNOWN	=>	9;

=head2 Object new

 Title        : new
 Usage        : my $o_err = new Error::ObjectException( $obj|$class_name, KEYWORD, -text => $msg);
 Prerequisite : it's better for an error to be catch !
 Object       : Constructor, initialize the object
 Returns      : the object.
 Args         :	none
 Globals      : none

=cut

sub new
{
	my $class = shift;
	my ($arg_un, $keyword) = (shift, shift);
	my %h_param = (
					@_
				);

	local $Error::Depth = $Error::Depth + 1;
# 	local $Error::Debug = 1;  # Enables storing of stacktrace

	my $tmp = ref( $arg_un );
	my ( $class_obj, $obj ) = ($tmp eq '') ? ($arg_un, undef) : ($tmp, $arg_un);

	my $o_err = $class->SUPER::new( -object => $obj,
									-class => $class_obj,
# 									-text => $text,
									%h_param);

	return $o_err;
}

sub Explain
{
	my $self = shift;
	
	# Construct message
	my $msg = $self->GetMsg() . $self->text();

	return $msg;
}

1;
