#
#       Sebastien.Letort@toulouse.inra.fr
#       Created: August 30, 2007
#       Last Updated: August 30, 2007
#       $Id: Article.pm 86 2007-10-10 08:05:41Z sletort $
#

package Biomoby::Article;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=pod

=head1 NAME

	Biomoby::Article

=head1 SYNOPSIS

	this class should not be instancied.

=head1 DESCRIPTION

	It's an interface for primary and secondary article in biomoby.

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 

=head1 METHODS

=cut

use strict;
use warnings;

use Data::Dumper;

## de lipmutils
#use Enum;
use General;
use base qw( LipmObject );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 86 $ =~ /\d+/g); $r[0]};
}

# fermeture
{

    # a redefinir dans les classes filles
    # my $_o_enum;
    sub GetEnum { }

=head2 Function _Init

	Title      :	_Init
	Usage      :	$article_son->SUPER::_Init( $name, $type, $descr );
	Prerequiste:    none
	Function   :	Initialize name, type and descr attributes.
	Returns    :	the changed object
	Args       :	$name,	the name of the article
	            	$type,	the type of the article, must belong to allowed value
	            	$descr,	the description of the article
	Error      :    none
	Globals    :    none

=cut

    sub _Init
    {
        my $self = shift;
        my ($name, $type, $mandatory, $descr) = @_;

        $self->{name}      = $name or return;
        $self->{type}      = $type;
        $self->{descr}     = $descr || 'no description';
        $self->{mandatory} = $mandatory || 0;

        return undef unless ($self->CheckType() == &TRUE);

        return $self;
    }

}

=head2 Function CheckType

	Title      :	CheckType
	Usage      :	$article->CheckType()
	Prerequiste:    The class must have declared a list of allowed value
	Function   :	check that the type attribute is in alloawed_value
	Returns    :	1 if succed, 0 otherwise
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub CheckType
{
    my $self = shift;

    my $ra_enum = $self->GetEnum();
    die "Error on GetEnum (Does the constructor ok ? does the GetEnum redefine ?)" unless defined $ra_enum;
	return (grep /^$self->{type}$/,@{$ra_enum});
    #return $o_enum->IsOk($self->{type});
}

=head2 Function ToString

	Title      :	ToString
	Usage      :	$article->ToString
	Prerequiste:    none
	Function   :	return the object as a string, for debug purpose
	Returns    :	a string containing all attributes
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub ToString
{
    my $self   = shift;
    my $string = '';

    while (my ($key, $val) = each(%$self))
    {
        if (defined($val) and ref($val) eq '') {$string .= "$key=$val\t";}
    }

    return $string;
}

=head2 Procedure SetCmd

	Title      :	SetCmd
	Usage      :	$art->SetCmd( $cmd )
	Prerequiste:    none
	Procedure  :	set the cmd attribute _format in the mobyle xml
	Args       :	$cmd,	the command that represent the article
	Error      :    none
	Globals    :    none

=cut

sub SetCmd
{
    my $self = shift;
    my ($cmd) = @_;

    $self->{cmd} = $cmd;

    return;
}

=head2 Procedure SetFilename

	Title      :	SetFilename
	Usage      :	$art->SetFilename( $filename )
	Prerequiste:    none
	Procedure  :	set the filename attribute
	Args       :	$filename,	the name of the file
	Error      :    none
	Globals    :    none

	Note       :	I'd like paramsfile and filenames to be mutually exclusive

=cut

sub SetFilename
{
    my $self = shift;
    my ($filename) = @_;

    $self->{filename} = $filename;

    return;
}

#	==	==	==	==	==	==	==	==	==	==	==	==	==	==	==
package Biomoby::ArticlesCollection;

=pod

=head1 NAME

	Biomoby::ArticlesCollection

=head1 SYNOPSIS

	my $collec = New Biomoby::ArticlesCollection();
	$collec->Add( New Biomoby::PrimArticle( @params1 ) );
	$collec->Add( New Biomoby::PrimArticle( @params2 ) );
	my @a_ids = $collec->GetIds();
	my $o_article = $collec->GetById(2);
	my $o_article = $collec->GetByName( 'lolo' );

	print $collect->ToString();

=head1 DESCRIPTION

	This class managed a group of articles.
	It provides method to add article, to retrieve one by its name or its id.

=cut

use strict;
use warnings;
use Data::Dumper;

=head2 Function New

	Title      :	New
	Usage      :	New Biomoby::ArticlesCollection( $class_name );
	Prerequiste:	Nowaday the only obligation for the class is to provide :
	            		- attribut 'name'
	            		- method 'ToString'
	            	Maybe later we'll check that $class_name belong to @Article::INC
	Function   :	constructor of a collection of articles,
	            	if a class name is passed, the collection will be homogene
	Returns    :	an object Biomoby::ArticlesCollection
	Args       :	$class_name,	the name of a class that inherite from Article
	Error      :    none
	Globals    :    none
	Note       :	Nowaday the only obligation for the class is to provide :
	            		- attribut 'name'
	            		- method 'ToString'

=cut

sub New
{
    my $class           = shift;
    my $self            = {};
    my ($article_class) = @_;

    $self->{class} = $article_class || 'Biomoby::Article';

    $self->{articles} = {};
    $self->{cur_id}   = 0;
    bless($self, $class);

    return ($self);
}

=head2 Procedure Add

	Title      :	Add
	Usage      :	$collec->Add( $o_article );
	Prerequiste:	$o_article must be an $self->{class} object
	Procedure  :	Add an article into the collection, give it an id
	Args       :	$o_article,	an object belonging to $self->{class} class
	Error      :    none
	Globals    :    none

=cut

sub Add
{
    my $self = shift;
    my ($o_article) = @_;

    return undef if (!$o_article->isa($self->{class}));

    my $r_id = \$self->{cur_id};
    ++$$r_id;

    # pour permettre un acces a l'id depuis l'objet
    # un autre moyen serait de se servir du nom comme de l'id ...
    $o_article->{id} = $$r_id;
    $self->{articles}{$$r_id} = $o_article;

    return;
}

=head2 Function GetById

	Title      :	GetById
	Usage      :	$o_article = $collec->GetById( $id )
	Prerequiste:	none
	Function   :	accessor to an identified article
	Returns    :	an object Biomoby::Article
	Args       :	$id,	the id of the article (given by GetIds)
	Error      :    none
	Globals    :    none

=cut

sub GetById
{
    my $self = shift;
    my ($id) = @_;

    return $self->{articles}{$id};
}

=head2 Function GetByName

	Title      :	GetByName
	Usage      :	$o_article = $collec->GetByName( $name )
	Prerequiste:	none
	Function   :	accessor to an identified article
	Returns    :	an object Biomoby::Article
	Args       :	$name,	the name of the article
	Error      :    none
	Globals    :    none

=cut

sub GetByName
{
    my $self = shift;
    my ($art_name) = @_;

    my $rh = $self->{articles};
## ! each n'est à utiliser que quand on parcourt tout le tableau
##	ici, on s'arrete avant et donc a l'appel suivant,
##	on ne recommence pas à zéro, mais là où on s'était arrêté.
    #	while( my ($id, $o_article) = each( %$rh ) )
    foreach my $id (keys %$rh)
    {
        my $o_article = $$rh{$id};
        return $o_article if ($o_article->{name} eq $art_name);
    }

    return undef;
}

=head2 Function GetIds

	Title      :	GetIds
	Usage      :	my @a_ids = $collec->GetIds();
	Prerequiste:	none
	Function   :	accessor to the ids of articles
	Returns    :	an array of ids
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetIds
{
    my $self = shift;

    my $rh_h = $self->{articles};
    return keys %$rh_h;
}

=head2 Function ToString

	Title      :	ToString
	Usage      :	$string = $collec->ToString();
	Prerequiste:	none
	Function   :	transform the object into a string
	Returns    :	a string
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub ToString
{
    my $self   = shift;
    my $string = "";

    foreach my $id (sort $self->GetIds)
    {
        $string .= "$id : " . $self->GetById($id)->ToString() . "\n";
    }

    return $string;
}

=head2 Function GetFilenames

	Title      :	GetFilenames
	Usage      :	@a_filenames = $collec->GetFilenames();
	Prerequiste:	none
	Function   :	give a list of filenames involved into this collection
	Returns    :	an array of string
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetFilenames
{
    my $self = shift;
    my @a_list;

    foreach my $id (sort $self->GetIds)
    {
        if (defined $self->{filename})
        {
            push(@a_list, $self->GetById($id)->{filename});
        }
    }

    return @a_list;
}

1;
