#
#       Sebastien.Letort@toulouse.inra.fr
#       Created: September 03, 2007
#       Last Updated: September 07, 2007
#       $Id: Object.pm 88 2007-10-11 15:41:10Z carrere $
#

package Biomoby::Object;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

	Biomoby::Object

=head1 SYNOPSIS

	my $object = New Biomoby::Object( $namespace, $type );

=head1 DESCRIPTION

	This class aims to facilitate deals with Biomoby object.
	It's an encapsulation of what could be done with ParamParser object.

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 

=head1 METHODS


=cut

use strict;
use warnings;

# fermeture : les variables privées ne sont pas visibles de l'extérieur
{
    #my $_o_enum_primitive_type = New Enum('Object', 'String', 'Integer', 'Float', 'Boolean', 'DateTime');

=head2 Function IsPrimitiveType

	Title      :	IsPrimitiveType
	Usage      :	Biomoby::Object::IsPrimitiveType( $val );
	Prerequiste:    none
	Function   :	check if $val is a primitive Biomoby object type
	Returns    :	a boolean
	Args       :	$val, the value to check
	Error      :    none
	Globals    :    none

=cut

    sub IsPrimitiveType
    {
        my ($val) = @_;
        my @a_primitives = ('Object', 'String', 'Integer', 'Float', 'Boolean', 'DateTime');
        return grep (/^$val$/,@a_primitives);
        #return $_o_enum_primitive_type->IsOk($val);
    }
}

=head2 Function New

	Title      :	New
	Usage      :	my $obj = New Biomoby::Object( $namespace, $type );
	Prerequiste:    none
	Function   :	Build a PrimaryArticle object
	Returns    :	a Object object
	Args       :	$namespace,	the namespace of the object
	            	$type,	the type of the object, must be known by Biomoby
	Error      :    none
	Globals    :    none

=cut

sub New
{
    my $class = shift;
    my $self  = {};

    my ($namespace, $type) = @_;

    $self->{ns} = $namespace;
    $self->{tp} = $type;

    # 	$self->{ id } = $id;

    bless($self, $class);

    return ($self);
}

=head2 Procedure AddFile

	Title      :	AddFile
	Usage      :	my $object = $obj->AddFile( $filename )
	Prerequiste:    none
	Procedure  :	add a filename for the object test
	Args       :	$filename
	Error      :    none
	Globals    :    none

=cut

sub AddFile
{
    my $self = shift;
    my ($filename) = @_;

    #tester l'existence ?
    $self->{file} = $filename;

    return;
}

=head2 Function ToString

	Title      :	ToString
	Usage      :	my $object = $obj->ToString()
	Prerequiste:    none
	Function   :	represente the object as a string, for debugging purpose
	Returns    :	a string representing the object
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub ToString
{
    my $self   = shift;
    my $string = "$self->{ns} et $self->{tp}\n";

    return $string;
}

=head2 Procedure WrapAndWriteObject

	Title      :	WrapAndWriteObject
	Usage      :	my $object = $obj->WrapAndWriteObject
						( $obj_id, $article_name, $test_dir )
	Prerequiste:    none
	Procedure  :	To launch test procedure, data must be wrapped into moby xml.
	            	if the object's type is not primitive, it is wrapped again into a moby:string content.
	Args       :	$obj_id, $article_name, $test_dir
	Error      :    none
	Globals    :    none

=cut

sub WrapAndWriteObject
{
    my $self = shift;
    my ($file, $obj_id, $article_name, $test_dir) = @_;

    my $type            = $self->{tp};
    my $output_filename = $test_dir . "$article_name.$obj_id.moby";
    my $fs_test         = new IO::File($output_filename, "w") or die "$!";

    my $page = "<moby:$type namespace='$self->{ns}'";
    $page .= " id='$obj_id' articleName='$article_name'>";

    # if the object is a primitive one _except 'Object', add the corresponding element
    if (!&IsPrimitiveType($type))
    {

        # if it is not a primitive, it is a moby:string content
        $page .= "\t<moby:String articleName='content' id='' namespace='' >";
    }

    # put data into CDATA if not an object
    if ('Object' ne $type) {$page .= "<![CDATA[";}

    # print the first part of the page
    print $fs_test $page;
    $page = '';

    # lecture des donnees du fichier
    if ('Object' ne $type)
    {
        my $fe_data = new IO::File($test_dir . $file, "r") or die "$!";

        while (my $line = <$fe_data>) {$page .= $line;}
        print $fs_test $page;

        $page = "]]>\n";

        if (!&IsPrimitiveType($type))
        {
            $page .= "\t</moby:String>\n";
        }
    }

    print $fs_test $page . "</moby:$type>";

    return;
}

##	--	--	--	--	--	--	--	--	--	--

=pod

=head1 NAME

	Biomoby::CollectionObject

=head1 SYNOPSIS

	my $collec = New Biomoby::CollectionObject();
	$collec->Add( New Biomoby::Object( @params1 ) );
	$collec->Add( New Biomoby::Object( @params2 ) );
	my @a_ids = $collec->GetIds();
	my $o_object = $collec->Get();

=head1 DESCRIPTION

	This class managed a group of objects.
	It provides method to add object, to retrieve one by its id.

=cut

package Biomoby::CollectionObject;

use strict;
use warnings;

=head2 Function New

	Title      :	New
	Usage      :	New Biomoby::CollectionObject( );
	Prerequiste:	none
	Function   :	constructor of a collection of objects,
	Returns    :	an object Biomoby::CollectionObject
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub New
{
    my $class = shift;
    my $self  = {};

    # 	my ( ) = @_;

    $self->{collection} = {};

    $self->{cur_id} = 0;
    bless($self, $class);

    return ($self);
}

=head2 Procedure Add

	Title      :	Add
	Usage      :	$collec->Add( $o_object );
	Prerequiste:	$o_object must be an Object : not check is done
	Procedure  :	Add an object into the collection, give it an id
	Args       :	$o_object,	an Object object
	Error      :    none
	Globals    :    none

=cut

sub Add
{
    my $self = shift;
    my ($o_object) = @_;

    my $r_id = \$self->{cur_id};
    $self->{collection}{++$$r_id} = $o_object;

    return;
}

=head2 Function Get

	Title      :	Get
	Usage      :	my $o_object = $collec->Get( $id );
	Prerequiste:	none
	Function   :	retrieve an object given its id
	Returns    :	the Object whose id is $id
	Args       :	$id the id of the object to retreive
	Error      :    none
	Globals    :    none

=cut

sub Get
{
    my $self = shift;
    my ($id) = @_;

    return $self->{collection}{$id};
}

=head2 Function GetIds

	Title      :	GetIds
	Usage      :	my @a_ids = $collec->GetIds();
	Prerequiste:	none
	Function   :	accessor to the ids of objects
	Returns    :	an array of ids
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetIds
{
    my $self = shift;

    my $rh_h = $self->{collection};
    return keys %$rh_h;
}

1;
