#
#       Sebastien.Letort@toulouse.inra.fr
#       Created: August 31, 2007
#       Last Updated: August 31, 2007
#       $Id: PrimaryArticle.pm 88 2007-10-11 15:41:10Z carrere $
#

package Biomoby::PrimaryArticle;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

	Biomoby::PrimaryArticle

=head1 SYNOPSIS

	my $prim = New Biomoby::PrimaryArticle(5, 'un', 'simple', 'input');

=head1 DESCRIPTION

	This class aims to facilitate access to Biomoby Input and Output.
	Its particularity is to have a collection of Biomoby Objects

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 

=head1 METHODS

=cut

use strict;
use warnings;

use Data::Dumper;

# heritage
use base qw(Biomoby::Article);
use Biomoby::Object;

# fermeture : les variables privées ne sont pas visibles de l'extérieur
{
#    my $_o_enum_type = New Enum('simple', 'collection');

=head2 Function GetEnum

	Title      :	GetEnum
	Usage      :	Biomoby::PrimaryArticle::GetEnum();
	Prerequiste:    none
	Function   :	Accessor to $_o_enum_type
	Returns    :	array ref
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

    sub GetEnum {return ['simple', 'collection'];}
}

=head2 Function New

	Title      :	New
	Usage      :	my $prim = New Biomoby::PrimaryArticle
	            					( $id, $name, $type, $direction, $o_objects );
	Prerequiste:    none
	Function   :	Build a PrimaryArticle object
	Returns    :	a PrimaryArticle object
	Args       :	$id,	the id of the article, will probably disappear
	            	$name,	the name of the article
	            	$type,	the type of the article, must belong to allowed value defined in array ref
	            	$direction,	'input' or 'output'
	            	$o_objects,	a collection of Object
	Error      :    none
	Globals    :    none

=cut

sub New
{
    my $class = shift;
    my $self  = {};
    bless($self, $class);

    my ($name, $type, $direction, $o_objects) = @_;

    $self->SUPER::_Init($name, $type, 1);

    # fn check type ?
    if (($direction ne 'input' and $direction ne 'output')) {return undef;}

    $self->{direction} = $direction;

    # 	$self->{ id } = $id;
    $self->{o_objects} = $o_objects;

    return ($self);
}

#uniquement pour un article simple ?
#plus tard peut être un iterateur

=head2 Function GetObject

	Title      :	GetObject
	Usage      :	my $object = $prim->GetObject()
	Prerequiste:    use only for a 'simple' article
	Function   :	return the only object of a simple article
	Returns    :	an object Object
	Args       :	none
	Error      :    none
	Globals    :    none
	
	Note       :	deprecated, use GetObjects or find another solution

=cut

sub GetObject
{
    my $self = shift;

    return undef if ('simple' ne $self->{type});

    return $self->{o_objects}->{collection}{1};

}

=head2 Function GetObjects

	Title      :	GetObjects
	Usage      :	my @a_objects = $prim->GetObjects()
	Prerequiste:    use only for a 'collection' article
	Function   :	return all the objects of the article
	Returns    :	an array of Object
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetObjects
{
    my $self = shift;

    my $rh_collection = $self->{o_objects}->{collection};

    return values %$rh_collection;
}

1;
