#
#       Sebastien.Letort@toulouse.inra.fr
#       Created: August 31, 2007
#       Last Updated: August 31, 2007
#       $Id: SecondaryArticle.pm 88 2007-10-11 15:41:10Z carrere $
#

package Biomoby::SecondaryArticle;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

	Biomoby::SecondaryArticle

=head1 SYNOPSIS

	my $prim = New Biomoby::SecondaryArticle('un entier', 'integer', 'nawak', 5);

=head1 DESCRIPTION

	This class aims to facilitate access to Biomoby parameters called secondary articles.

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 

=head1 METHODS

=cut

use strict;
use warnings;

use Data::Dumper;

# heritage
use base qw(Biomoby::Article);

# de lipmutils
#use Enum;

# fermeture : les variables privées ne sont pas visibles de l'extérieur
{
#    my $_o_enum_type = New Enum('integer', 'float', 'string', 'boolean', 'datetime');

=head2 Function GetEnum

	Title      :	GetEnum
	Usage      :	Biomoby::SecondaryArticle::GetEnum();
	Prerequiste:    none
	Function   :	Accessor to $_o_enum_type
	Returns    :	the Enum object
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

    sub GetEnum {return ['integer', 'float', 'string', 'boolean', 'datetime'];}
}

=head2 Function New

	Title      :	New
	Usage      :	my $prim = New Biomoby::SecondaryArticle
	            					( $name, $type, $descr, $position );
	Prerequiste:    none
	Function   :	Build a SecondaryArticle object
	Returns    :	a SecondaryArticle object
	Args       :	$name,	the name of the article
	            	$type,	the type of the article, must belong to allowed value defined in array ref
	            	$descr,	the description of the article
	            	$position,	the position of this article into the commande line
	Error      :    none
	Globals    :    none

=cut

sub New
{
    my $class = shift;
    my $self  = {};
    bless($self, $class);

    my ($name, $type, $mandatory, $descr, $position) = @_;

    $self->SUPER::_Init($name, $type, $mandatory, $descr);

    $self->{position} = $position;

    return ($self);
}

=head2 Procedure SetFilename

	Title      :	SetFilename
	Usage      :	$prim->SetFilename( $filename )
	Prerequiste:    none
	Procedure  :	set the filename attribute
	Args       :	$filename,	the filename related to this article
	Error      :    none
	Globals    :    none

=cut

sub SetFilename
{
    my $self = shift;
    my ($filename) = @_;

    $self->{filename} = $filename;

    return;
}

=head2 Procedure SetInfile

	Title      :	SetInfile
	Usage      :	$prim->SetInfile( $infile )
	Prerequiste:    none
	Procedure  :	set the infile attribute
	Args       :	$infile,	the infile related to this article
	Error      :    none
	Globals    :    none

	Note       :	There should be only one SetFilename
Je dois vérifier que si Filename est defini, paramsfile n'a pas lieu d'être et inversement.

=cut

sub SetInfile
{
    my $self = shift;
    my ($filename) = @_;

    $self->{infile} = $filename;

    return;
}

=head2 Procedure SetCtrlAndPrecond

	Title      :	SetCtrlAndPrecond
	Usage      :	$prim->SetCtrlAndPrecond( $ctrl, $precond )
	Prerequiste:    none
	Procedure  :	set the ctrl and precond attribute
	Args       :	$ctrl,	the command line that is used to set a value for this article
	            	$ctrl_msg,	displayed message for $ctrl exception or help
			$precond,	the command line that indicates if the article is needed or not
	Error      :    none
	Globals    :    none

=cut

sub SetCtrlAndPrecond
{
    my $self = shift;
    my ($ctrl, $ctrl_msg, $precond) = @_;

    $self->{ctrl}     = $ctrl;
    $self->{ctrl_msg} = $ctrl_msg;
    $self->{precond}  = $precond;

    return;
}

=head2 Procedure SetScale

	Title      :	SetScale
	Usage      :	$prim->SetScale( $min, $max, $inc )
	Prerequiste:    none
	Procedure  :	set the range of valid values
	Args       :	$min,	the minimum allowed
	            	$max,	the maximum allowed
	            	$inc,	the incremental step _not really used for now
	Error      :    none
	Globals    :    none

=cut

sub SetScale
{
    my $self = shift;
    my ($min, $max, $inc) = @_;

    $self->{min} = $min;
    $self->{max} = $max;
    $self->{inc} = $inc;

    return;
}

=head2 Procedure SetDefault

	Title      :	SetDefault
	Usage      :	$prim->SetDefault( $default, $ra_enum )
	Prerequiste:    none
	Procedure  :	set the default value and the enumeration of allowed value
	Args       :	$default,	the default value for the article
	            	$ra_enum, an array ref defining allowed values
	Error      :    none
	Globals    :    none

	Note       :	SetEnum should be part of setScale ?

=cut

sub SetDefault
{
    my $self = shift;
    my ($default, $ra_enum) = @_;

    $self->{default} = $default;
    $self->{ra_enum}  = $ra_enum;

    # verify that default is in enum if define
    if (defined $self->{ra_enum})
    {
        my $bool = grep (/^$default/,@{$self->{ra_enum}});
        $self->{default} = undef if (0 == $bool);
    }

    return;
}

#	==	==	==	==	==	==	==	==	==	==	==	==	==	==	==
package Biomoby::SecondaryArticlesCollection;

=pod

=head1 NAME

	Biomoby::SecondaryArticlesCollection

=head1 SYNOPSIS

	my $collec = New Biomoby::SecondaryArticlesCollection();
	$collec->Add( New Biomoby::SecondaryArticle( @params1 ) );
	$collec->Add( New Biomoby::SecondaryArticle( @params2 ) );
	my @a_ids = $collec->GetIds();
	my $o_article = $collec->GetById(2);
	my $o_article = $collec->GetByName( 'lolo' );

	print $collect->ToString();

	# what is in this class and not into its mother
	my @a_infiles = $collec->GetInfileNames();

=head1 DESCRIPTION

	This class managed a group of articles.
	It provides method to add article, to retrieve one by its name or its id.

=cut

use strict;
use warnings;
use Data::Dumper;

# heritage
use base qw(Biomoby::ArticlesCollection);

use General;    # pour IsNotNull

=head2 Function New

	Title      :	New
	Usage      :	New Biomoby::SecondaryArticlesCollection();
	Prerequiste:	none
	Function   :	constructor of a collection of secondary articles
	Returns    :	an object Biomoby::SecondaryArticlesCollection
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub New
{
    my $class = shift;
    my $self  = $class->SUPER::New('Biomoby::SecondaryArticle');

    bless($self, $class);

    return ($self);
}

=head2 Function GetInfileNames

	Title      :	GetInfileNames
	Usage      :	my @a_list = GetInfileNames();
	Prerequiste:	none
	Function   :	give a list of files into which params will be written
	Returns    :	an array of string
	Args       :	none
	Error      :    none
	Globals    :    none

	Note       :	infile et filename devraient etre reuni dans un seul parametre

=cut

sub GetInfileNames
{
    my $self = shift;
    my %h_list;

    foreach my $id ($self->GetIds)
    {
        my $filename = $self->GetById($id)->{infile};
        if (IsNotNull($filename))
        {
            $h_list{$filename} = 1;
        }
    }

    return keys %h_list;
}

1;
