#
#       Sebastien.Letort@toulouse.inra.fr
#       Created: September 03, 2007
#       Last Updated: September 03, 2007
#       $Id: Service.pm 88 2007-10-11 15:41:10Z carrere $
#

package Biomoby::Service;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

	Biomoby::Service

=head1 SYNOPSIS

	my $service = New Biomoby::Service( ... );
	$service->Register();
	$service->Deregister();
	$service->Test();

=head1 DESCRIPTION

	This class aims to facilitate deals with Biomoby service.
	It's an encapsulation of what could be done with ParamParser object.

=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 

=head1 METHODS

=cut

use strict;
use warnings;

use IO::File;

use Data::Dumper;

use base qw( LipmObject );
use ParamParser;
use General;
use Biomoby::PrimaryArticle;
use Biomoby::SecondaryArticle;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 88 $ =~ /\d+/g); $r[0]};
    die "Please define PMBBIN environnement variable before use of $0" if (!defined $ENV{PMBBIN});
    die "Please define PMBCFG environnement variable before use of $0" if (!defined $ENV{PMBCFG});
}

use constant TEST_DIR   => 'test/';
use constant KEY_INPUT  => 'iar';
use constant KEY_OUTPUT => 'oar';
use constant KEY_PARAM  => 'sar';

=head2 Function New

	Title      :	New
	Usage      :	my $prim = New Biomoby::Service
	            					( -conf => $conf_file, -registry => $registry );
	Prerequiste:    none
	Function   :	Build a Biomoby Service
	Returns    :	a biomoby service
	Args       :	$conf_file,	the paramParser conf file, _needed to use Carrere script of registration, may dispear in the future.
	            	$registry,	the central registry where your webservice will be/is registered
	Error      :    none
	Globals    :    none

=cut

sub _Init
{
    my $self = shift;
    my (%h_args) = @_;

    if (IsNotNull($h_args{-conf}))
    {    # test -f ?
        $self->{__cfg_file} = $h_args{-conf};
        my $o_pp = New ParamParser($h_args{-conf});
        $self->__InitPP($o_pp);
    }

    if (IsNotNull($h_args{-registry}))
    {
        $self->{registry} = $h_args{-registry};
    }

    #if empty -> throw ObjectException ?
    return ($self);
}

=head2 Procedure Register

	Title      :	Register
	Usage      :	$prim->Register( $verbose )
	Prerequiste:    use Carrere script
	Procedure  :	Register the service into a registry
	Args       :	$verbose,	to have debug information printed out
	Error      :    none
	Globals    :    none

=cut

sub Register
{
    my $self = shift;
    my ($verbose) = @_;

    my $register_cmd = $self->GetBinPath() . '/pmb_RegisterService.pl';
    $register_cmd .= $self->__GetRegisterOptions(1);

    if (1 == $verbose) {print STDERR "register commande : $register_cmd\n";}

    my @a_err = `$register_cmd`;
    print Dumper \@a_err;

    #print STDOUT \@a_err;

    return;
}

=head2 Procedure Deregister

	Title      :	Deregister
	Usage      :	$prim->Deregister( $verbose )
	Prerequiste:    use Carrere script
	Procedure  :	deregister the service from the registry
	Args       :	$verbose,	to have debug information printed out
	Error      :    none
	Globals    :    none

=cut

sub Deregister
{
    my $self = shift;
    my ($verbose) = @_;

    my $deregister_cmd = $self->GetBinPath() . '/pmb_DeregisterService.pl';
    $deregister_cmd .= $self->__GetRegisterOptions(1);
    $deregister_cmd .= " --remove";

    if (1 == $verbose) {print STDERR "deregister commande : $deregister_cmd\n";}

    my @a_err = `$deregister_cmd`;

    # pour moment je n'en fait rien de très utile, plus tard avec les exceptions ce sera mieux
    print Dumper \@a_err;

    #print STDOUT \@a_err;

    return;
}

=head2 Procedure Test

	Title      :	Test
	Usage      :	$prim->Test( $verbose )
	Prerequiste:    use Carrere script
	Procedure  :	launch the script test
	Args       :	$verbose,	to have debug information printed out
	Error      :    none
	Globals    :    none

=cut

sub Test
{
    my $self = shift;
    my ($verbose) = @_;

    $self->WrapData($verbose);

    my $test_cmd = $self->GetBinPath() . '/pmb_DebugWS.pl';
    $test_cmd .= $self->__GetRegisterOptions();

    if ($self->{o_inputs}->GetIds() > 0)
    {
        $test_cmd .= " --input_files=\"";
        my @a_input_ids = $self->{o_inputs}->GetIds();
        foreach my $id (@a_input_ids)
        {
            my $o_article = $self->{o_inputs}->GetById($id);

            my @a_input_ids_obj = $o_article->{o_objects}->GetIds();
            foreach my $id_obj (@a_input_ids_obj)
            {
                my $o_object = $o_article->{o_objects}->Get($id_obj);
                my @a_files = split (/\s+/,$o_object->{file});
                if ($#a_files > 0) #COLLECTION
                {
                    for (my $i = 0; $i <= $#a_files; $i ++)
                    {
                         my $obj_id = BuildObjectId($o_article, $id, $id_obj) . '#' . $i;
                         $test_cmd .= "$self->{ name }/" . &TEST_DIR . "$o_article->{ name }.$obj_id.moby,";
                    }
                    chop $test_cmd; #on enleve la derniere virgule
                }
                else #PAS COLLECTION
                {
                    my $obj_id = BuildObjectId($o_article, $id, $id_obj);
                    $test_cmd .= "$self->{ name }/" . &TEST_DIR . "$o_article->{ name }.$obj_id.moby ";
                }
            }
        }
        $test_cmd .= "\"";
    }

    if (-e "$self->{ name }/" . &TEST_DIR . 'parameters')
    {
        $test_cmd .= " --parameters=" . "$self->{ name }/" . &TEST_DIR . 'parameters';
    }

    if (-e "$self->{ name }/" . &TEST_DIR  . $self->{ name } . '.pmbtest')
    {
        $test_cmd .= " --pmbtest_file=" . "$self->{ name }/" . &TEST_DIR . $self->{ name } . '.pmbtest';
    }
    
    if (1 == $verbose) {print STDERR "test commande : $test_cmd\n";}

    my $err = `$test_cmd`;

    # pour moment je n'en fait rien de très utile, plus tard avec les exceptions ce sera mieux
#    print Dumper \@a_err;

    print STDOUT $err;
    return;
}

=head2 Function GetBinPath

	Title      :	GetBinPath
	Usage      :	$string = $service->GetBinPath();
	Prerequiste:	none
	Function   :	accessor to the playmoby bin path
	Returns    :	a string, the absolute ? path to the user folder
	Args       :	none
	Error      :    none
	Globals    :    none



=cut

sub GetBinPath
{
    my $self = shift;

    #	my $path = $self->{ ws_path };
    # je rajoute '/' à la fin car les scripts de Carrere écrase mon fichier de conf donc ma valeur
    #	my $user_path = $self->{ user_path } . '/';

    return $ENV{PMBBIN};
}

sub GetCfgPath
{
    my $self = shift;

    #	my $path = $self->{ ws_path };
    # je rajoute '/' à la fin car les scripts de Carrere écrase mon fichier de conf donc ma valeur
    #	my $user_path = $self->{ user_path } . '/';

    return $ENV{PMBCFG};
}

=head2 Function __GetRegisterOptions

	Title      :	__GetRegisterOptions
	Usage      :	$string = $service->__GetRegisterOptions( $register );
	Prerequiste:	none
	Function   :	write shared part of the command line for (De)Registration
	Returns    :	a string, the options for registration
	Args       :	$register,	a boolean, registration need more informations
	Error      :    none
	Globals    :    none

=cut

sub __GetRegisterOptions
{
    my $self = shift;
    my ($register) = @_;

    my $cfg_path = $self->GetCfgPath();

    my $options .= " --central_conf=$cfg_path/central/" . $self->{registry} . ".cfg";
    $options    .= " --service_conf=" . $self->{__cfg_file};

    if (defined $register and 1 == $register)
    {
        $options .= " --module=" . $self->{name} . '.pm';

        #		$options .= " --special_conf=" . $self->{__spe_file}	if( $self->{__spe_file} ne '' );
    }

    return $options;
}

=head2 Procedure __InitPP

	Title      :	__InitPP
	Usage      :	$string = $service->__InitPP( $src );
	Prerequiste:	none
	Procedure  :	build the service from a ParamParser cfg file
	Args       :	$o_pp ParamParser object
	Error      :    none
	Globals    :    none

=cut

sub __InitPP
{
    my $self = shift;
    my ($o_pp) = @_;

    # ce que devrait être cet objet sans o_pp
    # general
    $self->{ws_path}   = $o_pp->Get('ws_path');
    $self->{user_path} = $o_pp->Get('user_path');

    #registry a definir soit en parametre soit dans biomoby.cfg
    $self->{registry} = 'mobycentral' unless defined $self->{registry};

    # les infos du service
    $o_pp->SelectNameSpace('service_');
    $self->{descr}   = $o_pp->Get('description');
    $self->{contact} = $o_pp->Get('contact');
    $self->{name}    = $o_pp->Get('name');
    $self->{category}    = $o_pp->Get('category');
    $self->{type}    = $o_pp->Get('type');
    $self->{doc}     = $o_pp->Get('documentation');

    $o_pp->SelectNameSpace('');

    # les entrees-sorties
    $self->{o_inputs}  = &__GetPrimariesArticlesFromPP($o_pp, &KEY_INPUT);
    $self->{o_outputs} = &__GetPrimariesArticlesFromPP($o_pp, &KEY_OUTPUT);

    # les parametres
    $self->{o_2d_articles} = &__GetSecondariesArticlesFromPP($o_pp, &KEY_PARAM);

    # concernant le fichier executable
    $o_pp->SelectNameSpace('bin_');
    my $cmd         = $o_pp->Get('exe');
    my $binary_path = $o_pp->Get('path');
    $o_pp->SelectNameSpace('');

    #print STDERR $ENV{PATH};

    if (!-x $binary_path or !IsNotNull($binary_path))
    {
        my $msg =
          "$self->{ name } will probably not be executed because executable ($binary_path) is not executable.\n";
        $msg .= "\tYou should describe its path into $self->{ name }.cfg file (bin_path).\n";
        print STDERR $msg;
    }
    else
    {
        my @a_tmp = split('/', $binary_path);
        my $bin_name = pop @a_tmp;

        # on substitue le nom de l'executable par son chemin
        $cmd =~ s/$bin_name/$binary_path/;
        $self->{cmd} = $cmd;
    }

    return;
}

=head2 Procedure WrapData

	Title      :	WrapData
	Usage      :	$string = $service->WrapData( $verbose );
	Prerequiste:	current dir is service_name directory
	Procedure  :	for each object of each input article, will reformat files
	Args       :	$verbose,	to have debug infos
	Error      :    none
	Globals    :    none

=cut

sub WrapData
{
    my $self = shift;
    my ($verbose) = @_;

    # pour chaque article primaire
    my @a_articles = $self->{o_inputs}->GetIds();
    foreach my $id_art (@a_articles)
    {

        # pour chaque objet
        my $o_article    = $self->{o_inputs}->GetById($id_art);
        my $article_name = $o_article->{name};
        my @a_objects    = $o_article->{o_objects}->GetIds();
        foreach my $id_obj (@a_objects)
        {
            my $o_object = $o_article->{o_objects}->Get($id_obj);

            # /!\ != $id_obj
            my $obj_id = BuildObjectId($o_article, $id_art, $id_obj);

            # on enrobe le fichier associe
            
            #COLLECTIONS
            my @a_files = split (/\s+/,$o_object->{file});
            if ($#a_files > 0)
            {
                for (my $i = 0; $i <= $#a_files; $i ++)
                {
                    if (1 == $verbose) {print STDERR "will try to wrap $a_files[$i] into $article_name.$obj_id#$i.moby\n";}

                    chdir($self->{name});
                    $o_object->WrapAndWriteObject($a_files[$i],"$obj_id#$i", $article_name, &TEST_DIR);
                    chdir('..');

                }
            }
            else    #PAS COLLECTION
            {
                if (1 == $verbose) {print STDERR "will try to wrap $o_object->{file} into $article_name.$obj_id.moby\n";}

                chdir($self->{name});
                $o_object->WrapAndWriteObject($o_object->{file},$obj_id, $article_name, &TEST_DIR);
                chdir('..');
            }
        }
    }

    return;
}

# should be part of the PrimaryArticle ?
sub BuildObjectId
{
    my ($o_article, $id_art, $id_obj) = @_;

    my $keyword = ($o_article->{direction} eq 'input') ? &KEY_INPUT : &KEY_OUTPUT;
    my $obj_id = $keyword . "$id_art\_$id_obj";

    return $obj_id;
}

=head2 Function __GetPrimariesArticlesFromPP

	Title      :	__GetPrimariesArticlesFromPP
	Usage      :	$o_inputs_coll = &__GetPrimariesArticlesFromPP( $o_pp, $ns );
	Prerequiste:	none
	Function   :	parse the cfg file to build input and output articles
	Returns    :	an object, a collection of primary articles
	Args       :	$o_pp,	the param parser object
	            	$ns,	the "namespace" of the information to retrieve, in fact a keyword
	Error      :    none
	Globals    :    none

=cut

sub __GetPrimariesArticlesFromPP
{
    my ($o_pp, $ns) = @_;

    #input
    my $o_prim_articles = New Biomoby::ArticlesCollection('Biomoby::PrimaryArticle');
    my @a_articles      = $o_pp->GetKeys("^$ns\\d+_tp");

    foreach my $type_key (@a_articles)
    {

        # 1- isoler l'id de l'article
        my ($id) = $type_key =~ /(^$ns\d+_)tp$/;
        $o_pp->SelectNameSpace($id);

        # 2- trouver les infos de l'article
        my $name      = $o_pp->Get('nm');
        my $type      = $o_pp->Get('tp');
        my $cmd       = $o_pp->Get('cmd');
        my $file      = $o_pp->Get('filename');
        my $o_objects = &__GetObjectsForArticleFromPP($o_pp, $id);

        # 3- creer l'object
        my $direction = ($ns eq &KEY_INPUT) ? 'input' : 'output';
        my $o_prim_article = New Biomoby::PrimaryArticle($name, $type, $direction, $o_objects);
        $o_prim_article->SetCmd($cmd);
        $o_prim_article->SetFilename($file);
        $o_prim_articles->Add($o_prim_article);
    }
    $o_pp->SelectNameSpace();

    return $o_prim_articles;
}

=head2 Function __GetSecondariesArticlesFromPP

	Title      :	__GetSecondariesArticlesFromPP
	Usage      :	$o_inputs_coll = &__GetSecondariesArticlesFromPP( $o_pp, $ns );
	Prerequiste:	none
	Function   :	parse the cfg file to build a collection of secondary articles
	Returns    :	an object, a collection of secondary articles
	Args       :	$o_pp,	the param parser object
	            	$ns,	the "namespace" of the information to retrieve, in fact a keyword
	Error      :    none
	Globals    :    none

=cut

sub __GetSecondariesArticlesFromPP
{
    my ($o_pp, $ns) = @_;

    #input
    my $o_sec_articles = New Biomoby::SecondaryArticlesCollection();
    my @a_articles     = $o_pp->GetKeys("^$ns\\d+_name");

    foreach my $name_key (@a_articles)
    {

        # 1- isoler l'id de l'article
        my ($id) = $name_key =~ /(^$ns\d+_)name$/;
        $o_pp->SelectNameSpace($id);

        # 2- trouver les infos de l'article
        my $name      = $o_pp->Get('name');
        my $type      = $o_pp->Get('type');
        my $descr     = $o_pp->Get('description');
        my $pos       = $o_pp->Get('position');
        my $mandatory = $o_pp->Get('mandatory');

        # 3- creer l'object
        my $o_sec_article = New Biomoby::SecondaryArticle($name, $type, $mandatory, $descr, $pos);

        my $tmp = $o_pp->Get('filename');
        $o_sec_article->SetFilename($tmp) if ($tmp ne '');

        $tmp = $o_pp->Get('infile');
        $o_sec_article->SetInfile($tmp) if ($tmp ne '');

        $tmp = $o_pp->Get('cmd');
        $o_sec_article->SetCmd($tmp) if ($tmp ne '');

        $tmp = $o_pp->Get('precond');
        my $tmp2 = $o_pp->Get('ctrl');
        my $tmp3 = $o_pp->Get('ctrl_msg');
        if ($tmp ne '' or $tmp2 ne '') {$o_sec_article->SetCtrlAndPrecond($tmp2, $tmp3, $tmp);}

        $tmp  = $o_pp->Get('default');
        $tmp2 = $o_pp->Get('enum');
        if ($tmp2 ne '') 
        {
			my @a_list = split (/,\s*/, $tmp2);
			$tmp2 = \@a_list;
		}
        else             {$tmp2 = undef;}
        if ($tmp ne '' or defined $tmp2) {$o_sec_article->SetDefault($tmp, $tmp2);}

        $tmp  = $o_pp->Get('min');
        $tmp2 = $o_pp->Get('max');
        $tmp3 = $o_pp->Get('inc');
        if ($tmp ne '' and $tmp2 ne '' and $tmp3 ne '') {$o_sec_article->SetScale($tmp, $tmp2, $tmp3);}

        $o_sec_articles->Add($o_sec_article);
    }
    $o_pp->SelectNameSpace();

    return $o_sec_articles;
}

=head2 Function __GetObjectsForArticleFromPP

	Title      :	__GetObjectsForArticleFromPP
	Usage      :	$string = &__GetObjectsForArticleFromPP( $o_pp, $id_article );
	Prerequiste:	none
	Function   :	build a collection of object for a given article
	Returns    :	an CollectionObject object
	Args       :	$o_pp,	the paramparser object
	            	$id_article,	the id of the article to which the object belongs
	Error      :    none
	Globals    :    none

=cut

sub __GetObjectsForArticleFromPP
{
    my ($o_pp, $id_article) = @_;

    my @a_objects = $o_pp->GetKeys('otp');
    return if (0 == @a_objects);

    my $o_objects = New Biomoby::CollectionObject();

    foreach my $object (@a_objects)
    {

        # 1- isoler l'id de l'objet
        my ($id) = $object =~ /(^\d+_)otp$/;
        $o_pp->SelectNameSpace($id_article . $id);

        # 2- trouver les infos de l'objet
        my $namespace = $o_pp->Get('ons');
        my $type      = $o_pp->Get('otp');
        my $file      = $o_pp->Get('odat');

        # 3- ajouter un objet à la collection
        my $o_object = New Biomoby::Object($namespace, $type);
        $o_object->AddFile($file);
        $o_objects->Add($o_object);
    }
    $o_pp->SelectNameSpace();

    return $o_objects;
}

=head2 Function ToString

	Title      :	ToString
	Usage      :	$string = $service->ToString();
	Prerequiste:	none
	Function   :	convert the object into a string, for debug purpose
	Returns    :	a string, the object stringify
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub ToString
{
    my $self   = shift;
    my $string = "name=$self->{name}\ttype=$self->{type}\tdescr=$self->{descr}\n";
    $string .= "\n\tinput articles\n" . $self->{o_inputs}->ToString();
    $string .= "\n\toutput articles\n" . $self->{o_outputs}->ToString();
    $string .= "\n\tsecondary object\n" . $self->{o_2d_articles}->ToString();

    return $string;
}

=head2 Function GetCmd

	Title      :	GetCmd
	Usage      :	$string = $service->GetCmd();
	Prerequiste:	none
	Function   :	accessor for the cmd line
	Returns    :	the cmd line
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetCmd
{
    my $self = shift;

    return eval($self->{cmd});
}

=head2 Function GetPrebin

	Title      :	GetPrebin
	Usage      :	$string = $service->GetPrebin();
	Prerequiste:	none
	Function   :	Accessor to the prebin command
	Returns    :	the prebin command
	Args       :	none
	Error      :    none
	Globals    :    none

	Note       :	# a faire

=cut

sub GetPrebin
{
    my $self = shift;

    return '';
}

=head2 Function GetFilenames

	Title      :	GetFilenames
	Usage      :	@a_filenames = $service->GetFilenames();
	Prerequiste:	none
	Function   :	give the filenames involved in this service
	Returns    :	an array containing all filenames.
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetFilenames
{
    my $self = shift;
    my @a_list;

    #les fichiers d'entree des parametres
    push(@a_list, $self->{o_2d_articles}->GetInfileNames());

    return @a_list;
}

sub GetOutfileNames
{
    my $self = shift;
    my @a_list;

    push(@a_list, $self->{o_outputs}->GetFilenames());

    return @a_list;
}

1;
