package MobyleElement;

# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

MobyleElement

=head1 DESCRIPTION


=head1 AUTHORS

 Sebastien.Carrere@toulouse.inra.fr 
 Sebastien.Letort@toulouse.inra.fr 


=head1 METHODS

=cut

use strict;
use warnings;

use Data::Dumper;

use XML::Twig;
our @ISA = ("XML::Twig::Elt");    # héritage

use constant CMD       => 'cmd';
use constant ENTREE_1R => 'primary_article';
use constant ENTREE_2R => 'secondary_article';
use constant SORTIE    => 'output';
use constant RIEN      => '';
use constant LANG      => 'en';                  #en pour english criss

# le nom du hash qui va contenir les valeurs des parametres
# necessaire pour tester la 'precondition'
use constant HASH_PARAM => 'h_param';

#pas de constructeur, Twig s'en charge !

=head2 Function GetType

	Title      :	GetType
	Usage      :	$type = $elt->GetType( \$mandatory );
	Prerequiste:	none
	Function   :	return the type of the parameter regarding the elt's attributes
	Returns    :	a string constant CMD | SORTIE | ENTREE_1R | ENTREE_2R
	Args       :	$r_mandatory is set to 1 if this parameter is mandatory (and is not CMD)
	Error      :    none
	Globals    :    none

	Note       :	la nouvelle DTD donne l'attribut 'ismaininput'
	            	mais comme je ne suis pas sûr pour le moment
	            	qu'un MainInput soit sans precondition, je laisse comme tel.

=cut

sub GetType
{

    #	$elt doit etre un element 'parameter'
    my $self = shift;
    my ($r_mandatory) = @_;

    my $type;
    if ($self->GotThisAttributeTrue('iscommand')) {return CMD;}

    if ($self->GotThisAttributeTrue('ismandatory')) {$$r_mandatory = 1;}

    if (!$self->HavePrecond())
    {
        if ($self->GotThisAttributeTrue('isout')) {return SORTIE;}
        elsif (1 == $$r_mandatory) {return ENTREE_1R;}
    }

    # on filtre encore une fois
    if ($self->GotThisAttributeTrue('ismaininput')) {return ENTREE_1R;}

    # par defaut : un parametre
    return ENTREE_2R;
}

sub GetParameterElements
{

    #	$self doit etre un element parameter
    my $self = shift;
    my ($type, $mandatory) = @_;

    my $mask = RIEN;
    my %h_attr;

    # pour les parametres qui deviennent obligatoires si leur precond est verifie
    $h_attr{'mandatory'} = $mandatory;

    # le type biomoby

    #les entrees
    if ($type eq ENTREE_1R or $type eq SORTIE)
    {
        $self->GetPrimaryAttributes(\%h_attr);
        $mask = "mask";
    }
    elsif ($type eq ENTREE_2R)
    {
        $self->GetSecondaryAttributes(\%h_attr);
        $mask = "maskparam";
    }

    # la position
    my $tmp_elt = $self->first_child('argpos');
    my $pos = $h_attr{'position'} = (defined $tmp_elt) ? $tmp_elt->text : RIEN;
    $pos = 0 if ($type eq CMD);    # and $pos eq RIEN ?

    # la commande perl
    if (! defined $h_attr{'cmd'})
    {
    	$h_attr{'cmd'} = &GetPerlCmd($self);
    }

    # le fichier où écrire
  
    $tmp_elt = $self->first_child('paramfile');
    if (defined $tmp_elt) {$h_attr{'infile'} = $tmp_elt->text;}

    # print Dumper [$type, \%h_attr, $mask, $pos];

    return \%h_attr, $mask, $pos;
}

sub GetPerlCmd
{
    my $parameter_elt = shift();
    my $paramname     = $parameter_elt->first_child_text('name');
    #si la commande perl n'existe pas mais que la commandce python oui
    #faire une traduction
    #python -c 'value = "$value";print str ( " -o %s" % value)' 
    
    my $tmp_elt = $parameter_elt->first_child('format');
    my $perl_cmd = (defined $tmp_elt) ? $tmp_elt->GetText('code', 'proglang', 'perl') : RIEN;

    if (defined $tmp_elt)
    {
        if ((! defined $tmp_elt->GetText('code', 'proglang', 'perl')) || ($tmp_elt->GetText('code', 'proglang', 'perl') eq ''))
        {
            if (defined $tmp_elt->GetText('code', 'proglang', 'python') && ($tmp_elt->GetText('code', 'proglang', 'python') ne ''))
            {
                    my $python_cmd = $1;
                    my $python_eval_cmd = "'" . 'value = "$value";print (' . $python_cmd . ')' . "'";
                    print STDERR "WARNING: $paramname Perl command is not defined. Try to translate Python command: $python_cmd\n";
                    $perl_cmd = `python -c $python_eval_cmd`;
            }
            else
            {
                    $perl_cmd = (defined $tmp_elt) ? $tmp_elt->GetText('code', 'proglang', 'perl') : RIEN;
            }
        }
        else
        {
            $perl_cmd = (defined $tmp_elt) ? $tmp_elt->GetText('code', 'proglang', 'perl') : RIEN;
        }
    }
    return $perl_cmd;

}



sub GetPrimaryAttributes
{

    # $elt est un element parameter
    my $self = shift;
    my ($rh_attr) = @_;

    $$rh_attr{'filename'} = $self->GetFilename();
#
#Nouveau formalisme pour les sorties
#

	if ($$rh_attr{'filename'} =~ /^\$(\w+)$/)
	{
		my $outputname = $1;
		#tester si le parametre dont le nom est $paramname existe
		#getxpath pour aller recupere la commande;
            	my @a_parameters = $self->get_xpath("../parameter");
		foreach my $parameter_elt (@a_parameters)
		{
			my $paramname     = $parameter_elt->first_child_text('name');
			if ($outputname eq $paramname)
                        {
				$$rh_attr{'cmd'} = GetPerlCmd($parameter_elt);
				$$rh_attr{'secondary_not_to_print'} = $paramname;
			}
            
			else
			{
				next;
			}
			
		}
    }
    else
    {
     	foreach my $parameter_elt ($self->get_xpath("../parameter"))
	    {
		    my $paramname     = $parameter_elt->first_child_text('name');
            my $ishidden = $parameter_elt->{att}->{ishidden};
            if (defined $ishidden && ($ishidden == 1))
            {
                $$rh_attr{'secondary_not_to_print'} = $paramname;
                if ($paramname =~ /(\w+)_name/)
                {
                    $$rh_attr{'cmd'} = GetPerlCmd($parameter_elt) if ( $self->first_child_text('name') eq $1);
                }
            }
        }
   
    }


#
#
#
    $$rh_attr{'card'} = $self->GetCard();

    if  (($$rh_attr{'filename'} =~ /\*/) || ($$rh_attr{'card'} eq '1,n')) {$$rh_attr{'type'} = 'collection';}
    else                                 {$$rh_attr{'type'} = 'simple';}

    #en prevision de collection
    my @a_objects = ();
    push @a_objects, $self->GetPrimaryObjects();
    $$rh_attr{'objects'} = \@a_objects;

    return;
}

sub GetPrimaryObjects
{

    # $elt est un element parameter
    my $self = shift;

    my $type_obj = $self->GetBiomobyType();
    my $namespace_obj = $self->GetBiomobyNamespace();

    my %h_objet = ('ons' => $namespace_obj, 'otp' => $type_obj);

    # a ecrire si on a à faire à des collections heterogenes, sinon ok

    return \%h_objet;
}

sub GetSecondaryAttributes
{

    # $elt est un element parameter
    my $self = shift;
    my ($rh_attr) = @_;

    #descr
    $$rh_attr{'description'} = $self->GetText('prompt', 'lang', LANG);

    #valeur par defaut --et si plusieurs valeur par defaut cf clustalw ?
    my $tmp_elt = $self->first_child('vdef');
    $$rh_attr{'default'} = $tmp_elt->first_child_text('value') if defined $tmp_elt;

    #min-max
    $tmp_elt = $self->first_child('scale');
    $$rh_attr{'min'} = (defined $tmp_elt) ? $tmp_elt->GetScale('min') : RIEN;
    $$rh_attr{'max'} = (defined $tmp_elt) ? $tmp_elt->GetScale('max') : RIEN;

    #ctrl
    $tmp_elt = $self->first_child('ctrl');
    if (defined $tmp_elt)
    {
        $$rh_attr{'ctrl'} = $tmp_elt->GetText('code', 'proglang', 'perl');
        $tmp_elt = $tmp_elt->first_child('message');
        $$rh_attr{'ctrl_msg'} = $tmp_elt->GetText('text', 'lang', LANG);    #exception sinon
    }

    #precond
    $tmp_elt = $self->first_child('precond');
    if (defined $tmp_elt)
    {
        $$rh_attr{'precond'} = $tmp_elt->GetText('code', 'proglang', 'perl');
        my $mot = &HASH_PARAM;
        my $s = $$rh_attr{'precond'} =~ s/\$(\w+)/\$$mot\{ $1 }/g;
    }

    #type
    $$rh_attr{'type'} = $self->GetBiomobyType();

    #enum
    my $tmp = $self->first_child('vlist');
    $$rh_attr{'enum'} = $tmp->GetListChoice() if defined $tmp;

    #filename
    $$rh_attr{'filename'} = $self->GetFilename();

    return;
}

sub GetListChoice
{

    #	$elt est de type vlist
    my $self = shift;

    my @a_values;
    foreach my $tmp_elt ($self->children('velem'))
    {
        push @a_values, $tmp_elt->first_child_text('value');
    }

    return join ',', @a_values;
}

sub GetScale
{

    #	$elt est un element de type scale
    my $self = shift;
    my ($tag) = @_;

    my $tmp_elt = $self->first_child($tag);

    my $value_elt = $tmp_elt->first_child('value');
    return $value_elt->text if defined $value_elt;

    return $tmp_elt->GetText('code', 'proglang', 'perl');
}

=head2 Function GetText

	Title      :	GetText
	Usage      :	my $page = $elt->GetText( $child_name, $attr_name, $attr_val );
	Prerequiste:	none
	Function   :	return the content (#PCDATA) of the $child_name element
	            		which attribute name (=$attr_name)
	            		has the value $attr_val.
	            	It is used when an element have many child element but you want only one.
	Returns    :	a string
	Args       :	$child_name, the element name which contains the wanted text.
	            	$attr_name, the name of an attribute of the $child_name element
	            	$attr_val, the value of the $attr_name attribute.
	Error      :    none
	Globals    :    none

=cut

sub GetText
{
    my $self = shift;

    #	$elt est un element contenant un ou plusieurs elements text
    my ($child_name, $attr_name, $attr_val) = @_;

    my @a_elts = $self->children($child_name);
    return &RIEN if 0 == @a_elts;
    my $text;
    foreach my $text_elt (@a_elts)
    {
        my $value = $text_elt->att($attr_name);
        if (!defined $value)
        {

            # get the content but continue in the case of one element with the wanted attr.
            $text = $text_elt->text;
        }
        elsif ($text_elt->att($attr_name) eq $attr_val)
        {
            $text = $text_elt->text;
            last;
        }
    }

    # si pas d'attributs ou pas du bon type, on prend le premier
    $text = $a_elts[0]->text if !defined $text;

    return $text;
}

sub GotThisAttributeTrue
{
    my $self = shift;
    my ($att) = @_;

    # att() est une methode de twig::Element
    return (defined $self->att($att) && $self->att($att) eq '1');
}

sub HavePrecond
{

    #	$elt doit etre un element 'parameter'
    my $self = shift;

    my $precond_elt = $self->first_child('precond');
    return 0 if !defined $precond_elt;

    return 1;
}

sub GetFilename
{

    # $elt est un element parameter
    my $self = shift;

    my $fn_elt = $self->first_child('filenames');
    return '' if !defined $fn_elt;

    my $code = $fn_elt->GetText('code', 'proglang', 'perl');

    return $code;
}

#a function cause same code called twice
sub GetBiomobyType
{

    #	$self doit etre un element parameter
    my $self = shift;

    my $type_elt = $self->first_child('type');
    
    my $biomoby_elt = $type_elt->first_child('biomoby');
    my $biomoby = '';
    if (defined $biomoby_elt)
    {
        $biomoby     = $biomoby_elt->first_child_text('datatype');
    }
    $biomoby = 'NULL' if ($biomoby eq '');
    return $biomoby;
}

sub GetBiomobyNamespace
{

    #	$self doit etre un element parameter
    my $self = shift;

    my $type_elt = $self->first_child('type');
    
    my $biomoby_elt = $type_elt->first_child('biomoby');
    my $biomoby = '';
    if (defined $biomoby_elt)
    {
        $biomoby     = $biomoby_elt->first_child_text('namespace');
    }
    return $biomoby;
}

sub GetCard
{

    #	$self doit etre un element parameter
    my $self = shift;

    my $card_elt = $self->first_child('type');
    my $card     = $card_elt->first_child_text('card');
    $card = '1,1' if ($card eq '');
    return $card;
}


1;
